<?php
// -----------------------------------------------------------------------
//
// $Id: pgsql.php 8 2012-01-21 09:59:39Z root $
//
// Copyright (C) 2003-2012 Raoul Proena
// License: GNU GPL version 3 (see copying.txt file)
// Website: http://www.gnew.fr/
//
// -----------------------------------------------------------------------
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// -----------------------------------------------------------------------

class pgsql
{
    // ---------------------------
    var $host = SQL_HOST;
    var $port = SQL_PORT;
    var $user = SQL_USER;
    var $password = SQL_PASSWORD;
    var $database = SQL_DATABASE;
    // ---------------------------
    var $link_id;
    var $num_queries = 0;
    var $query_id;
    // ---------------------------

    function close()
    {
        return ($this->link_id) ? @pg_close($this->link_id) : false;
    }

    function connect()
    {
        $str = 'host=' . $this->host . ' port=' . $this->port . ' dbname=' . $this->database . ' user=' . $this->user . ' password=' . $this->password;
        $this->link_id = @pg_connect($str);
        return ($this->link_id) ? $this->link_id : $this->error('');
    }

    function error($sql)
    {
        if (!$this->link_id)
        {
            echo 'Connection to PostgreSQL server has failed.';
        }
        if (!$this->query_id)
        {
            printf('Error in query "%s".', $sql);
        }
        exit();
    }

    function fetch()
    {
        return ($this->query_id) ? @pg_fetch_array($this->query_id) : false;
    }

    function insert_id()
    {
        if ($this->query_id && $this->last_query_id != '')
        {
            if (preg_match('`^INSERT[\n ]INTO[\n ]([a-z_]+)`', $this->last_query_id, $table))
            {
                $seq = array(TABLE_ANSWERS => TABLE_ANSWERS . '_answer_id_seq',
                             TABLE_CATEGORIES => TABLE_CATEGORIES . '_category_id_seq',
                             TABLE_COMMENTS => TABLE_COMMENTS . '_comment_id_seq',
                             TABLE_NEWS => TABLE_NEWS . '_news_id_seq',
                             TABLE_POSTS => TABLE_POSTS . '_post_id_seq',
                             TABLE_QUESTIONS => TABLE_QUESTIONS . '_question_id_seq',
                             TABLE_SMILIES => TABLE_SMILIES . '_smiley_id_seq',
                             TABLE_USERS => TABLE_USERS . '_user_id_seq',
                             TABLE_VOTES => TABLE_VOTES . '_vote_id_seq');
                $last_id = @pg_query($this->link_id, 'SELECT currval(\'' . $seq[$table[1]] . '\') AS last_id');
                if (!$last_id)
                {
                    return false;
                }
                $insert_id = @pg_fetch_array($last_id, NULL, PGSQL_ASSOC);
                return ($insert_id) ? $insert_id['last_id'] : false;
            }
        }
        return false;
    }

    function num_queries()
    {
        return $this->num_queries;
    }

    function num_rows()
    {
        return ($this->query_id) ? @pg_num_rows($this->query_id) : false;
    }

    function query($sql = '')
    {
        if (!$this->connect())
        {
            return false;
        }
        elseif ($sql != '')
        {
            $this->num_queries++;
            $sql = preg_replace('`LIMIT ([0-9]+), ([0-9]+)`', 'LIMIT \\2 OFFSET \\1', $sql);
            $this->last_query_id = $sql;
            $this->query_id = @pg_query($this->link_id, $sql);
            if ($this->query_id)
            {
                return $this->query_id;
            }
            else
            {
                return $this->error($sql);
            }
        }
        else
        {
            return false;
        }
    }
}

?>