<?php
// -----------------------------------------------------------------------
//
// $Id: template.php 8 2012-01-21 09:59:39Z root $
//
// Copyright (C) 2003-2012 Raoul Proena
// License: GNU GPL version 3 (see copying.txt file)
// Website: http://www.gnew.fr/
//
// -----------------------------------------------------------------------
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// -----------------------------------------------------------------------

class template
{
    // ---------------------------
    var $root = '.';
    var $files = array();
    var $keys = array();
    var $values = array();
    // ---------------------------

    function error($str)
    {
        exit('Template error: ' . $str);
    }

    function parse($name, $block = '', $loop = false)
    {
        if (array_key_exists($name, $this->files))
        {
            // Replace variables
            static $output = '';
            $output .= preg_replace($this->keys, $this->values, $this->files[$name]);

            // Unset arrays
            unset($this->files);
            unset($this->keys);
            unset($this->values);

            if ($name == 'footer')
            {
                // Remove variables with no value
                $output = preg_replace('#{[^ \n\r\t}]+}#', '', $output);

                echo $output;
                unset($output);
            }
        }
        elseif ($loop)
        {
            if (array_key_exists($name, $this->values))
            {
                $str = preg_replace($this->keys, $this->values, $this->values[$name]);
                if (array_key_exists($block, $this->values))
                {
                    $this->set_var($block, $this->values[$block] . $str);
                }
                else
                {
                    $this->set_var($block, $str);
                }
            }
            else
            {
                $this->error('unable to parse "' . $name . '" block.');
                return false;
            }
        }
        else
        {
            $this->error('unable to parse "' . $name . '" file.');
            return false;
        }
        return true;
    }

    function set_block($parent, $name, $block)
    {
        if (array_key_exists($parent, $this->files))
        {
            $pattern = '`[ \t]*<!-- BEGIN ' . $name . ' -->\s*?\n?(\s*.*?\n?)\s*<!-- END ' . $name . ' -->\s*?\n?`s';
            if (preg_match_all($pattern, $this->files[$parent], $matches))
            {
                $this->files[$parent] = preg_replace($pattern, '{' . $block . '}', $this->files[$parent]);
                $this->set_var($name, $matches[1][0]);
            }
            else
            {
                $this->error('unable to set block.');
                return false;
            }
        }
        else
        {
            $this->error('unable to open parent file.');
            return false;
        }
        return true;
    }

    function set_file($name, $file)
    {
        if (!empty($file))
        {
            $handle = rtrim($this->root, '/') . '/' . $file;
            if ($fp = @fopen($handle, 'r'))
            {
                if (!empty($name))
                {
                    $this->files[$name] = fread($fp, filesize($handle));
                    fclose($fp);
                }
                else
                {
                    $this->error('function set_file(): $name argument is empty.');
                    return false;
                }
            }
            else
            {
                $this->error('unable to open "' . $handle . '" file.');
                return false;
            }
        }
        else
        {
            $this->error('function set_file(): $file argument is empty.');
            return false;
        }
        return true;
    }

    function set_var($name, $value = '')
    {
        if (!empty($name))
        {
            if (!is_array($name))
            {
                $this->keys[$name] = '`' . preg_quote('{' . $name . '}') . '`';
                $this->values[$name] = $value;
            }
            else
            {
                reset($name);
                while (list($k, $v) = each($name))
                {
                    $this->keys[$k] = '`' . preg_quote('{' . $k . '}') . '`';
                    $this->values[$k] = $v;
                }
            }
        }
        else
        {
            $this->error('function set_var(): $name argument is empty.');
            return false;
        }
        return true;
    }

    function template($root)
    {
        if (!is_dir($root))
        {
            $this->error('"' . $root . '" does not exist or is not a directory.');
            return false;
        }
        $this->root = $root;
        return true;
    }
}

?>