<?php
// -----------------------------------------------------------------------
//
// $Id: search.php 8 2012-01-21 09:59:39Z root $
//
// Copyright (C) 2003-2012 Raoul Proena
// License: GNU GPL version 3 (see copying.txt file)
// Website: http://www.gnew.fr/
//
// -----------------------------------------------------------------------
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// -----------------------------------------------------------------------

include('./../includes/common.php');

page_header($lang['NEWS_SEARCH_TITLE']);

// Clean and format string
$pattern = array('`[ ]+`', '`[?]+`');
$replace = array(' ', '?');
$str = preg_replace($pattern, $replace, trim($_GET['search']));
$str = chars2entities($str, ENT_QUOTES);

$search = '';
for ($i = 0; $i < strlen($str); $i++)
{
    if (preg_match('`[a-z0-9&#;]`i', $str{$i}))
    {
        $search .= $str{$i};
    }
    elseif ($str{$i} == ' ')
    {
        $search .= '%';
    }
    elseif ($str{$i} == '?')
    {
        $search .= '_';
    }
    else
    {
        $search .= '';
    }
}

// HTML entities
preg_match_all('`&[^;\s]+;`', $search, $entities);
$num_entities = $offset = 0;
for ($i = 0; $i < sizeof($entities[0]); $i++)
{
    $num_entities++;
    $offset += strlen($entities[0][$i]);
}

// Check string
if (!empty($search) && (strlen($search) - $offset + $num_entities) >= 2)
{
    $num_news = 0;
    $date_format = get_date_format();
    $date_offset = get_date_offset();
    $template->set_file('search', 'news/search.htpl');
    $template->set_block('search', 'NEWS_BLOCK', 'news');
    $sql->query('SELECT ' . TABLE_CATEGORIES . '.category_id, ' . TABLE_CATEGORIES . '.category_image, ' . TABLE_CATEGORIES . '.category_name, ' . TABLE_NEWS . '.news_comments, ' . TABLE_NEWS . '.news_date, ' . TABLE_NEWS . '.news_id, ' . TABLE_NEWS . '.news_source, ' . TABLE_NEWS . '.news_subject, ' . TABLE_NEWS . '.news_text, ' . TABLE_USERS . '.user_id, ' . TABLE_USERS . '.user_name
                 FROM ' . TABLE_CATEGORIES . ', ' . TABLE_NEWS . ', ' . TABLE_USERS . '
                 WHERE ' . TABLE_CATEGORIES . '.category_id = ' . TABLE_NEWS . '.category_id
                 AND ' . TABLE_CATEGORIES . '.category_level IN (\'0\', \'2\')
                 AND ' . TABLE_NEWS . '.news_active = \'1\'
                 AND ' . TABLE_NEWS . '.user_id = ' . TABLE_USERS . '.user_id
                 AND LOWER(' . TABLE_NEWS . '.' . $_GET['match'] . ') LIKE \'%' . strtolower($search) . '%\'
                 ORDER BY ' . $settings['news_order'] . ' DESC
                 LIMIT 0, ' . $settings['search_limit']);
    while ($table_news = $sql->fetch())
    {
        // Count news
        $num_news++;

        // Temporary tags to delimit search string - only elements outside HTML tags are highlighted
        $news_subject = '<!-- GENU_TEMP -->' . $table_news['news_subject'] . '<!-- GENU_TEMP -->';
        $news_text = '<!-- GENU_TEMP -->' . $table_news['news_text'] . '<!-- GENU_TEMP -->';

        $element = explode('%', $search);
        for ($i = 0; $i < count($element); $i++)
        {
            // Highlight elements
            if ($_GET['match'] == 'news_subject')
            {
                $news_subject = preg_replace('`([^&])(\b' . $element[$i] . '\b)([^;]){0,1}(?=[^>]*<)`i', '\\1<span class="newItem">\\2</span>\\3', $news_subject);
            }
            if ($_GET['match'] == 'news_text')
            {
                $news_text = preg_replace('`([^&])(\b' . $element[$i] . '\b)([^;]){0,1}(?=[^>]*<)`i', '\\1<span class="newItem">\\2</span>\\3', $news_text);
            }
        }

        // Remove temporary tags
        $news_subject = str_replace('<!-- GENU_TEMP -->', '', $news_subject);
        $news_text = str_replace('<!-- GENU_TEMP -->', '', $news_text);

        $news_date = date($date_format, ($table_news['news_date'] + $date_offset));
        if ($table_news['news_source'])
        {
            $news_source = undo_escape_sequences($table_news['news_source']);
            $news_source = sprintf($lang['NEWS_INDEX_SOURCE'], $news_source);
        }
        else
        {
            $news_source = '<!-- empty -->';
        }
        $news_text = undo_escape_sequences($news_text);
        $news_text = cut_text($news_text, './../news/index.php?news_id=' . $table_news['news_id']);
        $template->set_var(array('CATEGORY_ID' => $table_news['category_id'],
                                 'CATEGORY_IMAGE' => $table_news['category_image'],
                                 'CATEGORY_NAME' => $table_news['category_name'],
                                 'NEWS_COMMENTS' => $table_news['news_comments'],
                                 'NEWS_ID' => $table_news['news_id'],
                                 'NEWS_INDEX_COMMENT' => $lang['NEWS_INDEX_COMMENT'],
                                 'NEWS_INDEX_RELEASE' => sprintf($lang['NEWS_INDEX_RELEASE'], $table_news['user_id'], $table_news['user_name'], $news_date),
                                 'NEWS_INDEX_SEND' => $lang['NEWS_INDEX_SEND'],
                                 'NEWS_SOURCE' => $news_source,
                                 'NEWS_SUBJECT' => $news_subject,
                                 'NEWS_TEXT' => $news_text));
        $template->parse('NEWS_BLOCK', 'news', true);
        $news_exist = true;
    }
    if ($num_news > $settings['search_limit'])
    {
        error_template(sprintf($lang['NEWS_SEARCH_ERROR1'], $settings['search_limit']));
    }
    else
    {
        if (isset($news_exist))
        {
                $string = chars2entities(stripslashes($_GET['search']), ENT_QUOTES);
                $template->set_var(array('BACK_HOME' => $lang['BACK_HOME'],
                                         'NEWS_SEARCH_HEADER' => sprintf($lang['NEWS_SEARCH_HEADER'], $string, $num_news)));
                $template->parse('search');
        }
        else
        {
            error_template($lang['NEWS_SEARCH_ERROR2']);
        }
    }
}
else
{
    error_template($lang['NEWS_SEARCH_ERROR3']);
}

page_footer();

?>